import { Duration } from 'aws-cdk-lib';
import {
  BuildEnvironmentVariableType,
  BuildSpec,
  ComputeType,
  EventAction,
  FilterGroup,
  LinuxBuildImage,
  Project,
  Source,
} from 'aws-cdk-lib/aws-codebuild';
import { IVpc } from 'aws-cdk-lib/aws-ec2';
import { Role } from 'aws-cdk-lib/aws-iam';
import { Construct } from 'constructs';

import { DhiStack } from '../dhi-stack/dhi-stack';

export interface DwpPrE2eProps {
  repo: string;
  role: Role;
  secretFullArn: string | undefined;
  vpc?: IVpc;
}

export class DwpPrE2e extends Construct {
  public project: Project;

  constructor(scope: DhiStack, id: string, props: DwpPrE2eProps) {
    super(scope, id);

    const { repo, role, secretFullArn, vpc } = props;

    const projectName = `${scope.dhiTags.businessUnit}-${scope.dhiTags.application}-pr-e2e`;

    this.project = new Project(this, 'DwpPrE2e', {
      projectName,
      timeout: Duration.minutes(25),
      vpc,
      role,
      environment: {
        buildImage: LinuxBuildImage.STANDARD_7_0,
        privileged: true,
        computeType: ComputeType.MEDIUM,
        environmentVariables: {
          SECRETS: {
            type: BuildEnvironmentVariableType.SECRETS_MANAGER,
            value: secretFullArn,
          },
        },
      },
      source: Source.bitBucket({
        owner: 'dhigroupinc',
        repo,
        buildStatusName: 'DWP: E2E Test',
        webhookTriggersBatchBuild: true,
        webhookFilters: [
          FilterGroup.inEventOf(EventAction.PULL_REQUEST_CREATED).andBaseBranchIs('main'),
          FilterGroup.inEventOf(EventAction.PULL_REQUEST_UPDATED).andBaseBranchIs('main'),
        ],
      }),
      buildSpec: BuildSpec.fromObject({
        version: 0.2,
        batch: {
          'fast-fail': true,
          'build-list': [
            {
              identifier: 'Shard1',
              env: {
                variables: {
                  BUILD_ID: '1/4',
                },
              },
            },
            {
              identifier: 'Shard2',
              env: {
                variables: {
                  BUILD_ID: '2/4',
                },
              },
            },
            {
              identifier: 'Shard3',
              env: {
                variables: {
                  BUILD_ID: '3/4',
                },
              },
            },
            {
              identifier: 'Shard4',
              env: {
                variables: {
                  BUILD_ID: '4/4',
                },
              },
            },
          ],
        },
        phases: {
          pre_build: {
            commands: [`./scripts/secrets-to-env.sh "$SECRETS"`],
          },
          build: {
            commands: [
              `docker build --secret id=envLocal,type=file,src=.env.local --target pr-shard --build-arg BUILD_ID="$BUILD_ID" .`,
            ],
          },
        },
      }),
    });

    this.project.enableBatchBuilds();
  }
}
